@@ -30,7 +30,7 @@ limitations under the License.
 
 SEXP bsum(SEXP x, SEXP block_size)
 {
-  if(!(Rf_isLogical(x) || Rf_isNumeric(x)))
+  if(!Rf_isNumeric(x))
   {
     Rf_errorcall(R_NilValue, "argument 'x' must be a numerical vector or array");
   }
@@ -53,7 +53,7 @@ SEXP bsum(SEXP x, SEXP block_size)
 
   memset(b, 0, m * sizeof(double));
 
-  R_len_t n = Rf_xlength(x);
+  R_len_t n = XLENGTH(x);
 
   switch(TYPEOF(x))
   {
@@ -89,7 +89,7 @@ SEXP bsum(SEXP x, SEXP block_size)
   return y;
 }
 
-SEXP approx_gradients(SEXP graph, SEXP target, SEXP values, SEXP gradients, SEXP index, SEXP epsilon)
+SEXP approx_gradient(SEXP graph, SEXP target, SEXP node, SEXP index, SEXP epsilon)
 {
   if(!cg_is(graph, "cg_graph"))
   {
@@ -101,16 +101,6 @@ SEXP approx_gradients(SEXP graph, SEXP target, SEXP values, SEXP gradients, SEXP
     Rf_errorcall(R_NilValue, "argument 'target' must be a cg_node object");
   }
 
-  if(!Rf_isEnvironment(values))
-  {
-    Rf_errorcall(R_NilValue, "argument 'values' must be an environment");
-  }
-
-  if(!Rf_isEnvironment(gradients))
-  {
-    Rf_errorcall(R_NilValue, "argument 'gradients' must be an environment");
-  }
-
   if(!Rf_isNumeric(index))
   {
     Rf_errorcall(R_NilValue, "argument 'index' must be a numeric scalar");
@@ -121,27 +111,11 @@ SEXP approx_gradients(SEXP graph, SEXP target, SEXP values, SEXP gradients, SEXP
     Rf_errorcall(R_NilValue, "argument 'epsilon' must be a numeric scalar");
   }
 
-  int k = Rf_asInteger(index);
-
-  double eps = Rf_asReal(epsilon);
-
-  if(eps < 0)
-  {
-    Rf_errorcall(R_NilValue, "argument 'epsilon' must be non-negative");
-  }
-
   int target_index;
 
   SEXP target_value = R_NilValue;
 
-  SEXP target_symbol = cg_node_symbol(target);
-
-  PROTECT_WITH_INDEX(target_value = Rf_findVarInFrame(values, target_symbol), &target_index);
-
-  if(target_value == R_UnboundValue)
-  {
-    Rf_errorcall(R_NilValue, "cannot find value of node '%s'", cg_node_name(target));
-  }
+  PROTECT_WITH_INDEX(target_value = cg_node_value(target), &target_index);
 
   if(!Rf_isReal(target_value))
   {
@@ -150,82 +124,64 @@ SEXP approx_gradients(SEXP graph, SEXP target, SEXP values, SEXP gradients, SEXP
     cg_node_set_value(target, target_value);
   }
 
-  if(k < 1 || k > Rf_xlength(target_value))
+  int k = Rf_asInteger(index);
+
+  if(k < 1 || k > XLENGTH(target_value))
   {
-    Rf_errorcall(R_NilValue, "cannot differentiate node '%s' at index %d", cg_node_name(target), k);
+    Rf_errorcall(R_NilValue, "cannot differentiate node '%s' at index %d",
+                 cg_node_name(target), k);
   }
 
-  SEXP dep = PROTECT(cg_graph_backward_dep(graph, target));
+  int node_index;
 
-  R_len_t n = Rf_xlength(dep);
+  SEXP node_value = R_NilValue;
+
+  PROTECT_WITH_INDEX(node_value = cg_node_value(node), &node_index);
+
+  if(!Rf_isReal(node_value))
+  {
+    REPROTECT(node_value = Rf_coerceVector(node_value, REALSXP), node_index);
+
+    cg_node_set_value(node, node_value);
+  }
+
+  int n = XLENGTH(node_value);
+
+  SEXP grad = PROTECT(Rf_allocVector(REALSXP, n));
+
+  double *x = REAL(node_value);
+  double *y = REAL(grad);
+
+  double eps = Rf_asReal(epsilon);
 
   for(int i = 0; i < n; i++)
   {
-    SEXP node = VECTOR_ELT(dep, i);
+    x[i] += eps;
 
-    if(cg_node_type(node) == CGPRM)
-    {
-      int node_index;
+    cg_graph_forward(graph, target);
 
-      SEXP node_value = R_NilValue;
+    REPROTECT(target_value = cg_node_value(target), target_index);
 
-      SEXP node_symbol = cg_node_symbol(node);
+    double t1 = REAL(target_value)[k - 1];
 
-      PROTECT_WITH_INDEX(node_value = Rf_findVarInFrame(values, node_symbol), &node_index);
+    x[i] -= 2 * eps;
 
-      if(node_value == R_UnboundValue)
-      {
-        Rf_errorcall(R_NilValue, "cannot find value of node '%s'", cg_node_name(node));
-      }
+    cg_graph_forward(graph, target);
 
-      if(!Rf_isReal(node_value))
-      {
-        REPROTECT(node_value = Rf_coerceVector(node_value, REALSXP), node_index);
+    REPROTECT(target_value = cg_node_value(target), target_index);
 
-        cg_node_set_value(node, node_value);
-      }
+    double t2 = REAL(target_value)[k - 1];
 
-      R_len_t m = Rf_xlength(node_value);
+    x[i] += eps;
 
-      SEXP grad = PROTECT(Rf_allocVector(REALSXP, m));
-
-      double *x = REAL(node_value);
-      double *y = REAL(grad);
-
-      for(int j = 0; j < m; j++)
-      {
-        x[j] += eps;
-
-        cg_graph_run(graph, target, values);
-
-        REPROTECT(target_value = Rf_findVarInFrame(values, target_symbol), target_index);
-
-        double y1 = REAL(target_value)[k - 1];
-
-        x[j] -= 2 * eps;
-
-        cg_graph_run(graph, target, values);
-
-        REPROTECT(target_value = Rf_findVarInFrame(values, target_symbol), target_index);
-
-        double y2 = REAL(target_value)[k - 1];
-
-        x[j] += eps;
-
-        y[j] = (y1 - y2) / (2 * eps);
-      }
-
-      SHALLOW_DUPLICATE_ATTRIB(grad, node_value);
-
-      Rf_defineVar(node_symbol, grad, gradients);
-
-      UNPROTECT(2);
-    }
+    y[i] = (t1 - t2) / (2 * eps);
   }
 
-  cg_graph_run(graph, target, values);
+  SHALLOW_DUPLICATE_ATTRIB(grad, node_value);
 
-  UNPROTECT(2);
+  cg_graph_forward(graph, target);
 
-  return gradients;
+  UNPROTECT(3);
+
+  return grad;
 }
